/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.charts;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleEdge;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.TreeMap;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.plot.Marker;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;

public class HistoryChart extends BaseChart {
  
  private XYPlot plot;
  private XYLineAndShapeRenderer renderer;
  private TimeSeriesCollection timeSeries;
  private DateAxis domainAxis;
  private NumberAxis rangeAxis;  

  private TreeMap<String, TimeSeries> listSeries;  
  private HashMap<Integer, String> listIndexLegend;
  private ArrayList<Color> listColor;  
  
  public HistoryChart( int width, int height ){
    super( width, height );
    fillColors();
    
    listIndexLegend = new HashMap<Integer, String>();
    listSeries = new TreeMap<String, TimeSeries>();      
    
    plot = new XYPlot();
    timeSeries = new TimeSeriesCollection();

    configureDomainAxis();
    configureRangeAxis();
    configureRenderer();

    plot.setDomainAxis( domainAxis );
    plot.setRangeAxis( rangeAxis );
    plot.setRangeAxisLocation(AxisLocation.TOP_OR_RIGHT);
    plot.setDataset( timeSeries );
  }  
  
  private void fillColors(){
    listColor = new ArrayList<Color>();
    listColor.add( SERIE1_COLOR );
    listColor.add( SERIE1_COLOR );
    listColor.add( SERIE2_COLOR );
    listColor.add( SERIE2_COLOR );
    listColor.add( SERIE3_COLOR);
    listColor.add( SERIE3_COLOR);
  }
  
  private void configureDomainAxis() {
    domainAxis = new DateAxis();
    // Minimum range : little more than one day (in millisecond)
    domainAxis.setAutoRangeMinimumSize( 86400000.0*1.1 );
    domainAxis.setUpperMargin( 0.1 );
  }

  private void configureRangeAxis() {
    rangeAxis = new NumberAxis();
    rangeAxis.setAutoRangeIncludesZero(false);
    rangeAxis.setNumberFormatOverride( new DecimalFormat( "0'%'" ) );
    rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
    rangeAxis.setAutoRangeMinimumSize(2.0);
    rangeAxis.setUpperMargin( 0.1 );
    rangeAxis.setLowerMargin( 0.1 );
  }  
  
  private void configureRenderer(){
    renderer = new XYLineAndShapeRenderer();
    renderer.setBaseShapesVisible( false );
    plot.setRenderer( renderer );     
  }
  
  public void initLegend(Integer id, String legend){
    listIndexLegend.put( id, legend ); 
    TimeSeries series = new TimeSeries( legend );
    timeSeries.addSeries( series );
    listSeries.put( legend, series );
    
    int index = listIndexLegend.size()-1;    
    if ( index == 0 ) {
      renderer.setSeriesStroke( index, new BasicStroke( 4.0f ) );        
    } else {
      if (index%2==0) {
        renderer.setSeriesStroke( index, new BasicStroke( 2.5f ) );
      } else {
        renderer.setSeriesStroke( index, getDashedStroke(2.5f) );
      }
    }
    renderer.setSeriesPaint( index, listColor.get( index ) );    
  }
  
  public void addMeasure( Double value, Date date, Integer id ) {
    String legend = listIndexLegend.get( id );
    listSeries.get( legend ).addOrUpdate( new Day( date ), value );
  }
  
  public void addLabel(Date date, String label) throws ParseException{
    Day d = new Day( date );
    double millis = d.getFirstMillisecond();
    Marker marker = new ValueMarker(millis);
    marker.setLabel(label);
    marker.setLabelAnchor(RectangleAnchor.TOP_RIGHT);
    marker.setLabelTextAnchor(TextAnchor.TOP_LEFT);
    Color c = new Color(191, 191, 191);
    marker.setLabelPaint(c);
    marker.setPaint(c);
    marker.setStroke(new BasicStroke(1.5f));
    plot.addDomainMarker(marker);
  }  

  protected BufferedImage getChartImage() throws IOException {
    JFreeChart chart = new JFreeChart( null, TextTitle.DEFAULT_FONT, plot, true );
    configureChart(chart, RectangleEdge.LEFT);
    return super.getBufferedImage( chart );
  }
}